import json

from si_tests.deployments.utils import file_utils, commons, helpers


def generate_ssl_certificates(os_manager, timeout):
    local_domain = helpers.get_local_domain(os_manager)
    commons.LOG.info("Prepare ssl config")
    certs_path = file_utils.get_context("certs")
    csr_yaml_path = file_utils.join(certs_path, "api-csr.yaml")
    csr_config = file_utils.get_yaml_content(csr_yaml_path)

    csr_config["CN"] = "*.openstack.svc.{}".format(local_domain)

    csr_config["hosts"].append("*.openstack.svc.{}".format(local_domain))

    csr_config_path = "api-csr.json"
    with open(file_utils.join(certs_path, csr_config_path), "w") as f:
        json.dump(csr_config, f)

    commons.LOG.info("Install cfssl tools")
    executor = commons.get_local_executor()
    cfssl_base = ("https://artifactory.mcp.mirantis.net"
                  "/artifactory/binary-dev-kaas-local"
                  "/openstack/bin/utils/cfssl")
    for app in ["cfssl", "cfssljson"]:
        result = executor.execute(
            f"curl -L {cfssl_base}/{app} -o {app} && chmod +x {app}",
            cwd=certs_path)

        assert result.exit_code == 0, result

    commons.LOG.info("Generate ssl certificates")
    result = executor.execute(
        "./cfssl gencert -ca=ca.crt -ca-key=ca.key {} |"
        "./cfssljson -bare \"api\"".format(csr_config_path),
        cwd=certs_path)

    commons.LOG.info("%s", result.stderr)
    assert result.exit_code == 0, result
