import time

from si_tests import settings
from si_tests.deployments.utils import commons, file_utils


def check_get_refapp_node_identifier(openstack_client_manager,
                                     collect=False,
                                     requests_count=50):
    hosts = set()
    response_count = 0
    for counter in range(requests_count):
        host = openstack_client_manager.get_refapp_node_identifier()
        if host:
            hosts.add(host)
            response_count += 1
        if len(hosts) == 3 and not collect:
            break
        time.sleep(1)
    else:
        if collect:
            return requests_count, response_count
        assert False, "App did not have 3 distinct GET responses " \
                      "from APIs within 50 retries"


def check_create_refapp_record(openstack_client_manager,
                               collect=False,
                               requests_count=50):
    records_file = settings.OPENSTACK_REFAPP_RECORDS_FILE
    hosts = set()
    records = list()
    response_count = 0
    for counter in range(requests_count):
        record = openstack_client_manager.create_refapp_record()
        if record:
            response_count += 1
            hosts.add(record["host"])
            records.append(record)

            file_utils.save_to_yaml(records,
                                    records_file,
                                    mode='a')

        if len(hosts) == 3 and not collect:
            break
        time.sleep(1)
    else:
        if collect:
            return requests_count, response_count
        assert False, "App did not have 3 distinct POST responses " \
                      "from APIs within 50 retries"


def check_get_refapp_record_by_id(openstack_client_manager,
                                  collect=False):
    records_file = settings.OPENSTACK_REFAPP_RECORDS_FILE
    records = file_utils.get_yaml_content(
        records_file,
        file_not_found_fail=False) or []
    request_count = len(records)
    response_count = 0

    for record in records:
        record_by_id = openstack_client_manager.get_refapp_record_by_id(
            record_id=record['id'])
        if collect:
            if record == record_by_id:
                response_count += 1
        else:
            assert record == record_by_id
    return request_count, response_count


def check_refapp(openstack_client_manager):
    """Check working capacity of RefApp.
        - Check already created records
        - Check that RefApp did have 3 distinct GET responses
        - Check that RefApp did have 3 distinct POST responses
        - Save created records.
    """
    commons.LOG.info("Check get RefApp records by id if "
                     "file with records exists.")
    check_get_refapp_record_by_id(openstack_client_manager)
    commons.LOG.info("Check GET RefApp requests.")
    check_get_refapp_node_identifier(openstack_client_manager)
    commons.LOG.info("Check POST RefApp requests.")
    check_create_refapp_record(openstack_client_manager)
    commons.LOG.info("Refapp check finished.")


def collect_refapp_statistic(openstack_client_manager):
    # Set retries to stop this process if I broke something here)))
    retries = settings.COLLECT_REFAPP_STATISTIC_RETRIES

    commons.LOG.info("Start refapp statistic collection.")
    for i in range(retries):
        get_host_req, get_host_resp = check_get_refapp_node_identifier(
            openstack_client_manager, collect=True, requests_count=3)

        create_req, create_resp = check_create_refapp_record(
            openstack_client_manager, collect=True, requests_count=1)

        id_req, id_resp = check_get_refapp_record_by_id(
            openstack_client_manager, collect=True)
        commons.LOG.warning("RefApp collected requests/responses counts:\n"
                            "Get host requests/responses: %s/%s\n"
                            "Create record requests/responses: %s/%s\n"
                            "Get record by ID requests/responses: %s/%s",
                            get_host_req, get_host_resp, create_req,
                            create_resp, id_req, id_resp)

        commons.LOG.warning("RETRY %s", i)
