import exec_helpers
import yaml
import json
from packaging import version

from si_tests import settings
from si_tests import logger
import si_tests.utils.templates as utils

LOG = logger.logger


def test_openstack_provider_e2e(
        kaas_manager, mgmt_os_provider_e2e_prerequisites):
    name = settings.OPENSTACK_PROVIDER_E2E_POD_NAME
    ns = settings.OPENSTACK_PROVIDER_E2E_NAMESPACE

    controller_pods = kaas_manager.api.api_core.list_namespaced_pod(
        namespace='kaas',
        label_selector='app.kubernetes.io/instance=openstack-provider')
    controller_pod = controller_pods.to_dict()['items'][0]
    kaas_version =\
        controller_pod['spec']['containers'][0]['image'].split(':')[1]

    if version.parse(kaas_version) < version.parse('0.2.37'):
        image_version = '0.2.34'
    else:
        image_version = settings.OPENSTACK_PROVIDER_E2E_IMAGE_VERSION\
                        or kaas_version

    secret_name = mgmt_os_provider_e2e_prerequisites.secret_name
    options = {
        'KUBECONFIG_SECRET_NAME': secret_name,
        'OPENSTACK_PROVIDER_E2E_SA': name,
        'OPENSTACK_PROVIDER_E2E_IMAGE':
            settings.ARTIFACTORY_URL + '/' +
            settings.OPENSTACK_PROVIDER_E2E_IMAGE + ':' + image_version
    }
    templates = utils.render_template(
        settings.OPENSTACK_PROVIDER_E2E_POD_YAML, options)
    LOG.debug(templates)
    json_body = json.dumps(yaml.load(templates, Loader=yaml.SafeLoader))
    pod = kaas_manager.api.pods.create(name=name,
                                       namespace=ns,
                                       body=json.loads(json_body))
    pod.wait_phase('Running')
    pod.wait_test(filepath='./test_*', timeout=3600)

    pod.cp_from_pod(source_dir=settings.OPENSTACK_PROVIDER_E2E_REPORTS_DIR)

    exec_helpers.Subprocess().execute("env", verbose=True)

    reports = exec_helpers.Subprocess().execute(
        "ls -1 artifacts/junit_*").stdout

    if reports:
        exec_helpers.Subprocess().check_call(
            "junitparser merge {0}/junit_*"
            " {0}/openstack_provider_e2e_report.xml && "
            "rm {0}/junit_*"
            .format(settings.ARTIFACTS_DIR))

    pod.delete()

    # Checking all pods are in consistent state after test run
    ns = kaas_manager.get_namespace(namespace=settings.TARGET_NAMESPACE)
    cluster = ns.get_cluster(settings.TARGET_CLUSTER)
    cluster.check.check_k8s_pods()
