#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import logger
from si_tests import settings

from si_tests.managers import bootstrap_manager

LOG = logger.logger


class TestProvisionEquinixRegionalClusters():

    def test_deploy_equinix_regional(self, kaas_manager,
                                     openstack_client, show_step):
        """Deploy Regional cluster on KaaS mgmt

        Scenario:
            1. Pass Region var and run bootsrap.sh deploy_regional
            2. Check result of deployment
        """

        bootstrap = bootstrap_manager.BootstrapManager()
        # step 001
        show_step(1)
        bootstrap.step_004_deploy_kaas_regional(
            openstack_client,
            region=settings.EQUINIX_REGION)

        # step 002
        show_step(2)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks(openstack_client)

        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()

        regional_cluster.store_k8s_artifacts()

        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_actual_expected_pods(timeout=1200)
        regional_cluster.provider_resources.save_artifact()
