#    Copyright 2019 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import settings, logger

from si_tests.managers import bootstrap_manager

LOG = logger.logger


class TestProvisionBmCluster():
    def test_bootstrap_kaas_bm(self, openstack_client, show_step):
        """Bootstrap KaaS_BM on OpenStack VM

        Scenario:
            1. Erase previous KaaS environment and BM node, if exists
            2. Create a stack with a node0
            3. Prepare a node0 and upload git repos
            4. Prepare for kaas-bm install, using ansible playbooks
            5. Install kaas-bm mgmt cluster using ansible playbooks
            6. Wait for all pods to be Running and Ready
            7. Check all expected pods exists
            8. Download bootstrap artifacts
            9. Check LDAP integration if enabled
        """

        bootstrap = bootstrap_manager.BootstrapManager()

        # step 001
        if settings.KEEP_ENV_BEFORE:
            LOG.info('Skip erase due KEEP_ENV_BEFORE flag is set')
        else:
            if openstack_client._current_stack:
                # Erase the stack with the seed node
                show_step(1)
                bootstrap.step_001_erase_env_before(openstack_client)
            else:
                LOG.warning("Can't erase environment due "
                            "seed node stack is absent")

        # Create a stack with a seed node
        show_step(2)
        bootstrap.step_002_provision_seed_vm(openstack_client)
        show_step(3)
        bootstrap.step_003_prepare_bm_seed_node(openstack_client)
        show_step(4)
        bootstrap.step_004_prepare_for_kaas_bm_cluster(openstack_client)
        show_step(5)
        bootstrap.step_004_deploy_kaas_bm_mgmt_cluster(openstack_client)
        show_step(6)
        bootstrap.step_006_wait_for_pods()
        show_step(7)
        if settings.RUN_POD_CHECKS:
            bootstrap.step_006_postdeployment_checks(openstack_client,
                                                     check_svc=True)
        show_step(8)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client)
        if settings.KEYCLOAK_LDAP_ENABLED:
            show_step(9)
            bootstrap.check_keystone_ldap_integration()

    def test_bootstrap_kaas_bm_physical(self, openstack_client, show_step):
        """Bootstrap KaaS_BM on OpenStack VM

        Scenario:
            1. Erase previous KaaS environment and BM node, if exists
            2. Create a stack with a node0
            3. Prepare a node0 and upload git repos
            4. Install kaas-bm cluster using ansible playbooks
            5. Wait for all pods to be Running and Ready
            6. Check that all expected pods exists
            7. Download bootstrap artifacts
            8. Check LDAP integration if enabled
        """

        bootstrap = bootstrap_manager.BootstrapManager()

        # step 001
        if settings.KEEP_ENV_BEFORE:
            LOG.info('Skip erase due KEEP_ENV_BEFORE flag is set')
        else:
            if openstack_client._current_stack:
                # Erase the stack with the seed node
                show_step(1)
                bootstrap.step_001_erase_env_before(openstack_client)
            else:
                LOG.warning("Can't erase environment due "
                            "seed node stack is absent")

        # Create a stack with a seed node
        show_step(2)
        bootstrap.step_002_provision_seed_vm(openstack_client)

        show_step(3)
        bootstrap.step_003_prepare_bm_seed_node(openstack_client)

        show_step(4)
        bootstrap.step_004_deploy_kaas_bm_cluster_physical(openstack_client,
                                                           settings.ENV_NAME)
        show_step(5)
        bootstrap.step_006_wait_for_pods()

        show_step(6)
        if settings.RUN_POD_CHECKS:
            bootstrap.step_006_postdeployment_checks(openstack_client,
                                                     check_svc=True)

        show_step(7)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client)

        if settings.KEYCLOAK_LDAP_ENABLED:
            show_step(8)
            bootstrap.check_keystone_ldap_integration()
