#    Copyright 2019 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import settings
from si_tests import logger

from si_tests.managers.kubeadm_manager import KubeadmManager

LOG = logger.logger


class TestProvisionBmSingle():
    def test_bootstrap_kaas_bm(self, openstack_client, show_step):
        """Bootstrap KaaS_BM on OpenStack VM

        Scenario:
            1. Erase previous KaaS environment and BM node, if exists
            2. Create a stack with node0
            3. Prepare node0 and upload git repos
            4. Install kaas-bm single node using ansible playbooks
            5. Install helm charts and wait for baremetal host is ready
        """

        manager = KubeadmManager()

        # step 001
        if settings.KEEP_ENV_BEFORE:
            LOG.info('Skip erase due KEEP_ENV_BEFORE flag is set')
        else:
            if openstack_client._current_stack:
                # Erase the stack with the seed node
                show_step(1)
                manager.step_001_erase_env_before(openstack_client)
            else:
                LOG.warning("Can't erase environment due "
                            "seed node stack is absent")

        # Create a stack with a seed node
        show_step(2)
        manager.step_002_provision_seed_vm(openstack_client)

        show_step(3)
        manager.step_003_prepare_bm_seed_node(openstack_client)

        show_step(4)
        manager.step_004_deploy_kaas_bm_single(openstack_client)

        show_step(5)
        manager.step_005_test_bm_helm_charts(openstack_client)
