import yaml
import json
import pytest
from si_tests import settings
from si_tests import logger

LOG = logger.logger


def test_kaas_bm_pods(kaas_manager, namespace=None):
    bm_pod_selector = [
        "app.kubernetes.io/name==kaas-bm-httpd",
        "app.kubernetes.io/name==kaas-bm-mariadb",
    ]
    ns = namespace or settings.BM_DEFAULT_NS
    for selector in bm_pod_selector:
        pods = [pod.name for pod in
                kaas_manager.api.pods.list(
                    namespace=ns, label_selector=selector)]
        assert any(pods) is True, \
            "No pod(s) matching {} were found on the cluster".format(selector)

        for pod_name in pods:
            pod = kaas_manager.api.pods.get(name=pod_name, namespace=ns).read()
            for container in pod.status.container_statuses:
                assert container.ready, \
                    "One of the container - {} in {} pod is not ready"\
                    .format(container.get("Name"), pod_name)


def test_kaas_bm_deployments(kaas_manager):
    ns = settings.BM_DEFAULT_NS
    bm_deploys_list = ['ironic',
                       'ironic-inspector',
                       'dnsmasq',
                       'baremetal-operator',
                       'baremetal-provider']

    for deploy_name in bm_deploys_list:
        deploy = kaas_manager.api.deployments.\
            get(name=deploy_name, namespace=ns)
        assert deploy.is_ready, \
            "Deployment {} is not ready".format(deploy_name)


def test_kaas_bm_metallb_pods(kaas_manager):
    ns = settings.BM_METALLB_NS
    pod_list = kaas_manager.api.pods.list(namespace=ns,
                                          label_selector="app==metallb")
    assert len(pod_list) != 0,\
        "Pods in {} namespace were not found".format(ns)
    for pod in pod_list:
        phase = kaas_manager.api.pods.status(name=pod.name, namespace=ns)
        phase = phase.status.phase
        assert phase == 'Running',\
            "Pod {} is not running. State is: {}".format(pod.name, phase)


def test_kaas_bm_actions(kaas_manager):
    ns_name = settings.BM_TEST_NS
    sec_template = settings.BM_SEC_TEMPLATE
    bmh_template = settings.BM_BMH_TEMPLATE
    if not bmh_template:
        pytest.skip("BMH test template was not found. Skipping")
    ns_ispresent = kaas_manager.api.namespaces.present(name=ns_name)
    if ns_ispresent:
        pytest.skip("{} namespace is already present. Skipping")
    with open(bmh_template) as template:
        yaml_body = yaml.load(template, Loader=yaml.SafeLoader)
        bmh_name = yaml_body['metadata'].get('name', '')
    if bmh_name == '':
        pytest.skip("BMH template doesn't contain name")
    ns = kaas_manager.create_namespace(ns_name)
    with open(sec_template) as template:
        json_body = json.dumps(yaml.load(template, Loader=yaml.SafeLoader))
    kaas_manager.api.secrets.\
        create(namespace=ns_name, body=json.loads(json_body))
    with open(bmh_template) as template:
        json_body = json.dumps(yaml.load(template, Loader=yaml.SafeLoader))
    kaas_manager.api.kaas_baremetalhosts\
        .create(namespace=ns_name, body=json.loads(json_body))
    ns.wait_baremetalhosts_statuses(wait_status='ready',
                                    interval=10,
                                    retries=30)
    bmh = kaas_manager.api.kaas_baremetalhosts\
        .get(namespace=ns_name, name=bmh_name)
    bmh.delete()
    ns.delete()
