import exec_helpers
import yaml
import json
import pytest

from si_tests import settings
from si_tests import logger
import si_tests.utils.templates as utils
from si_tests.utils import helpers

LOG = logger.logger


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
@pytest.mark.usefixtures("store_cluster_description")
def test_sl(kaas_manager, stacklight_prerequisites, keycloak_ip, _):
    name = settings.STACKLIGHT_TEST_POD_NAME
    ns = settings.STACKLIGHT_TEST_NAMESPACE

    keycloak_address = settings.KEYCLOAK_URL or ("https://" + keycloak_ip)

    cluster_name = settings.TARGET_CLUSTER

    secret_name = stacklight_prerequisites.secret_name

    target_namespace = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    target_cluster = target_namespace.get_cluster(cluster_name)
    kubectl_client = target_cluster.k8sclient

    helm_release = target_cluster.get_helmbundle()

    sl_version = [helpers.get_chart_version(x) for x
                  in helm_release.data['spec']['releases']
                  if x['name'] == 'stacklight']

    sl_test_image_dict = settings.STACKLIGHT_TEST_IMAGE_DICT
    if sl_version[0] in sl_test_image_dict:
        sl_version[0] = sl_test_image_dict[sl_version[0]]

    artifacts_region_map = settings.ARTIFACTS_REGION_MAPPING
    cdn_region = settings.CDN_REGION
    sl_image_base = artifacts_region_map[cdn_region]["images_base_url"]
    sl_image = settings.STACKLIGHT_TEST_IMAGE or sl_image_base + \
        "/stacklight/stacklight-pytest:" + sl_version[0]
    secrets = {}
    svc_list = ['prometheus', 'alerta', 'alertmanager', 'grafana']
    if target_cluster.logging_enabled():
        LOG.info("Logging is enabled on this cluster")
        svc_list.append('kibana')
    for svc in svc_list:
        field_selector = 'metadata.name=iam-proxy-{}-config'.format(svc)
        LOG.info(f"Checking configmap with selector {field_selector}")
        cm = kubectl_client.api_core.list_namespaced_config_map(
            namespace='stacklight',
            field_selector=field_selector)
        if cm.items:
            cm = cm.items[0]
            encryption_key = \
                yaml.load(cm.data['config.yaml'],
                          Loader=yaml.SafeLoader)['encryption-key']
        else:
            raise Exception("Cannot find configmap for {} proxy".format(svc))
        secrets['IAM_PROXY_{}_SECRET'.format(svc.upper())] = encryption_key
    options = {
        'KEYCLOAK_URL': keycloak_address,
        'KUBECONFIG_SECRET_NAME': secret_name,
        'STACKLIGHT_TEST_SA': name,
        'STACKLIGHT_TEST_POD_NAME': name,
        'STACKLIGHT_TEST_IMAGE': sl_image
    }
    options.update(secrets)
    templates = utils.render_template(settings.STACKLIGHT_TEST_POD_YAML,
                                      options)
    LOG.debug(templates)

    json_body = json.dumps(yaml.load(templates))
    pod = stacklight_prerequisites.parent_api.pods.create(
        name=name, namespace=ns, body=json.loads(json_body))
    pod.wait_phase('Running')
    pod.wait_test(filepath='./test_*', timeout=18600)

    pod.cp_from_pod(source_dir=settings.STACKLIGHT_TEST_REPORTS_DIR)

    exec_helpers.Subprocess().check_call(
        "test -f artifacts/report.xml && "
        "mv artifacts/report.xml artifacts/lma_{0}_report.xml"
        .format(cluster_name))

    pod.delete()

    # Checking all pods are in consistent state after test run
    # exclude conformance ns from ns_list for check pods
    ns_list = [namespace.name for namespace in kubectl_client.namespaces.list()
               if 'conformance' not in namespace.name]
    LOG.info("ns for check: {0}".format(ns_list))
    target_cluster.check.check_k8s_pods(target_namespaces=ns_list)
