#!/usr/bin/env python3

import os
import sys

sys.path.append(os.getcwd())
try:
    from si_tests import settings
    from si_tests.managers.kaas_manager import Manager
except ImportError:
    print("ImportError: Run the application from the si-tests directory or "
          "set the PYTHONPATH environment variable to directory which contains"
          " ./si_tests")
    sys.exit(1)


def main():
    if not settings.KUBECONFIG_PATH or not os.path.isfile(
            settings.KUBECONFIG_PATH):
        print("Please set KUBECONFIG environment variable whith"
              "the path to the kubeconfig file for KaaS management cluster")
        return 11

    kaas_manager = Manager(kubeconfig=settings.KUBECONFIG_PATH)

    cr_list = \
        kaas_manager.api.kaas_clusterreleases.list_raw().to_dict()['items']
    crs = [x for x in cr_list
           if 'lcmType' in x['spec'] and
           x['spec']['lcmType'] == 'ucp' and
           x['metadata']['name'].startswith("mke")]
    ucp_version = []
    for cr in crs:
        machinetypes_params = cr['spec']['machineTypes']['control'] + \
                              cr['spec']['machineTypes']['worker']
        ucp_tag = ''
        for ctl in machinetypes_params:
            if 'ucp_tag' in ctl['params'].keys():
                if not ucp_tag:
                    ucp_tag = ctl['params']['ucp_tag']
                else:
                    if ucp_tag != ctl['params']['ucp_tag']:
                        break
        else:
            if ucp_tag:
                ucp_version.append(ucp_tag)

    print('\n'.join(ucp_version))


if __name__ == '__main__':
    sys.exit(main())
